<?php

namespace AmazingCat\ZipGate\Helper;

use AmazingCat\ZipGate\Model\ResourceModel\ZipCode\CollectionFactory;
use AmazingCat\ZipGate\Model\ZipCodeFactory;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Session\SessionManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Store\Api\StoreCookieManagerInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

class ZipCodeHelper extends AbstractHelper
{
    const ZIPCODE_COOKIE_NAME = 'zipgate_code';
    /**
     * @var \Magento\Framework\Stdlib\CookieManagerInterface
     */
    protected $cookieManager;

    /**
     * @var \Magento\Framework\App\Http\Context
     */
    protected $httpContext;

    /**
     * @var StoreCookieManagerInterface
     */
    protected $storeCookieManager;

    /**
     * @var StoreRepositoryInterface
     */
    protected $storeRepository;

    /**
     * @var ZipCodeFactory
     */
    protected $zipCodeFactory;

    /**
     * @var \AmazingCat\ZipGate\Model\ResourceModel\ZipCode
     */
    protected $zipCode;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var CollectionFactory
     */
    protected $zipCodeCollectionFactory;

    /**
     * @var CookieMetadataFactory
     */
    protected $cookieMetadataFactory;

    /**
     * @var SessionManagerInterface
     */
    protected $sessionManager;

    /**
     * @var EncryptorInterface
     */
    protected $encryptor;

    /**
     * ZipCodeHelper constructor.
     * @param Context $context
     * @param \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager
     * @param \Magento\Framework\App\Http\Context $httpContext
     * @param SessionManagerInterface $sessionManager
     * @param StoreManagerInterface $storeManager
     * @param CookieMetadataFactory $cookieMetadataFactory
     * @param StoreCookieManagerInterface $storeCookieManager
     * @param CollectionFactory $zipCodeCollectionFactory
     * @param ZipCodeFactory $zipCodeFactory
     * @param \AmazingCat\ZipGate\Model\ResourceModel\ZipCode $zipCode
     * @param StoreRepositoryInterface $storeRepository
     * @param EncryptorInterface $encryptor
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager,
        \Magento\Framework\App\Http\Context $httpContext,
        SessionManagerInterface $sessionManager,
        StoreManagerInterface $storeManager,
        CookieMetadataFactory $cookieMetadataFactory,
        StoreCookieManagerInterface $storeCookieManager,
        CollectionFactory $zipCodeCollectionFactory,
        ZipCodeFactory $zipCodeFactory,
        \AmazingCat\ZipGate\Model\ResourceModel\ZipCode $zipCode,
        StoreRepositoryInterface $storeRepository,
        EncryptorInterface $encryptor
    ) {
        parent::__construct($context);
        $this->cookieManager = $cookieManager;
        $this->httpContext = $httpContext;
        $this->storeCookieManager = $storeCookieManager;
        $this->storeRepository = $storeRepository;
        $this->zipCodeFactory = $zipCodeFactory;
        $this->zipCode = $zipCode;
        $this->storeManager = $storeManager;
        $this->zipCodeCollectionFactory = $zipCodeCollectionFactory;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->sessionManager = $sessionManager;
        $this->encryptor = $encryptor;
    }

    /**
     * @param string $zip
     * @return ZipCodeHelper
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Stdlib\Cookie\CookieSizeLimitReachedException
     * @throws \Magento\Framework\Stdlib\Cookie\FailureToSendException
     */
    public function setZipCode(string $zip)
    {
        $zip = $this->encryptor->encrypt($zip);
        $metadata = $this->cookieMetadataFactory
            ->createPublicCookieMetadata()
            ->setDuration(172800) // 2 days
            ->setPath($this->sessionManager->getCookiePath())
            ->setDomain($this->sessionManager->getCookieDomain());
        $this->cookieManager->setPublicCookie(self::ZIPCODE_COOKIE_NAME, $zip, $metadata);
        return $this;
    }

    /**
     * @return string|null
     */
    public function getZipCode()
    {
        return $this->encryptor->decrypt($this->cookieManager->getCookie(self::ZIPCODE_COOKIE_NAME));
    }

    /**
     * @return $this
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Stdlib\Cookie\FailureToSendException
     */
    public function unsetZipCode()
    {
        $this->cookieManager->deleteCookie(
            self::ZIPCODE_COOKIE_NAME,
            $this->cookieMetadataFactory
                ->createCookieMetadata()
                ->setPath($this->sessionManager->getCookiePath())
                ->setDomain($this->sessionManager->getCookieDomain())
        );
        return $this;
    }

    /**
     * @param string $zip
     * @return \Magento\Store\Api\Data\StoreInterface
     * @throws NotFoundException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Stdlib\Cookie\CookieSizeLimitReachedException
     * @throws \Magento\Framework\Stdlib\Cookie\FailureToSendException
     * @throws \Magento\Store\Model\StoreIsInactiveException
     * @throws \Throwable
     */
    public function setStoreViewByCode(string $zip)
    {
        $this->setZipCode($zip);
        $zipCodeCollection = $this->zipCodeCollectionFactory->create();
        $zipCodeDto = $zipCodeCollection->addFieldToFilter('zip_code', $zip)->getFirstItem();

        if ($zipCodeDto->getStoreviewId() == null) {
            $this->unsetZipCode();
            throw new NotFoundException(__("Zip with code {$this->getZipCode()} not found"));
        }

        try {
            // Change current store
            $store = $this->storeRepository->getActiveStoreById($zipCodeDto->getStoreviewId());
            $this->httpContext->setValue(
                Store::ENTITY,
                $store->getCode(),
                $this->storeRepository->getActiveStoreById(Store::DEFAULT_STORE_ID)->getCode()
            );
            $this->storeCookieManager->setStoreCookie($store);
            return $store;
        } catch (\Throwable $e) {
            // Fallback to default store
            $defaultStore = $this->storeRepository->getActiveStoreById(Store::DEFAULT_STORE_ID);
            $this->httpContext->setValue(
                Store::ENTITY,
                $defaultStore->getCode(),
                $defaultStore->getCode()
            );
            $this->storeCookieManager->setStoreCookie($defaultStore);
            throw $e;
        }
    }

    /**
     * @return \AmazingCat\ZipGate\Model\ZipCode
     */
    public function getZipCodeDto()
    {
        return $this->zipCodeFactory->create();
    }

    public function getZipCodeResource()
    {
        return $this->zipCode;
    }

    /**
     * @return StoreManagerInterface
     */
    public function getStoreManager()
    {
        return $this->storeManager;
    }
}
